@@ -17,6 +17,7 @@
 #include <netdb.h>
 #include <sys/socket.h>
 #include <errno.h>
+#include <popt.h>
 
 #include "firestarter.h"
 #include "util.h"
@@ -33,6 +34,8 @@
 #include "xpm/statustopped.xpm"
 
 gboolean NETFILTER;
+gboolean CONSOLE;
+gboolean NO_APPLET;
 
 GtkWidget *window;
 GtkWidget *panelvbox;
@@ -61,17 +64,17 @@ appbar_update (char *state)
 	static int currentstate;
 
 	if (state == "stop") {
-		gnome_appbar_set_default (GNOME_APPBAR (appbar),
+		if (!NO_APPLET) gnome_appbar_set_default (GNOME_APPBAR (appbar),
 			_("Firewall stopped"));
 		currentstate = 1;
 	}
 	else if (state == "start") {
-		gnome_appbar_set_default (GNOME_APPBAR (appbar),
+		if (!NO_APPLET) gnome_appbar_set_default (GNOME_APPBAR (appbar),
 			_("Firewall running"));
 		currentstate = 2;
 	}
 	else if (state == "halt") {
-		gnome_appbar_set_default (GNOME_APPBAR (appbar),
+		if (!NO_APPLET) gnome_appbar_set_default (GNOME_APPBAR (appbar),
 			_("All traffic halted"));
 		currentstate = 3;
 	}
@@ -139,6 +142,28 @@ hide_window_cb (GtkWidget *window, GdkEventButton* bevent)
 	}
 }
 
+/* [ root_alert ]
+ * check if the users is root, if not display an alert message
+ */
+gint
+root_alert (void)
+{
+	gint uid;
+
+	if ((uid = getuid ())) {
+		if (!CONSOLE)
+		{
+			GtkWidget *dialog=gnome_error_dialog (_(
+				"You must be root to manipulate the firewall."));
+			gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
+		}
+		else
+			g_print (_("You must be root to manipulate the firewall.\n"));
+	}
+
+	return uid;
+}
+	
 /* [ stop_firewall ]
  * Flushes, zeroes and sets all policies to accept
  */
@@ -149,12 +174,8 @@ stop_firewall (void)
 	gchar *sbin;
 	gchar *command = NULL;
 
-	if (geteuid ()) {
-		GtkWidget *dialog=gnome_error_dialog (_(
-			"You must be root to manipulate the firewall."));
-		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
+	if (root_alert ())
 		return;
-	}
 
 	if (gnome_config_get_bool ("/firestarter/Druid/locatesbins=FALSE"))
 		if (NETFILTER)
@@ -213,12 +234,8 @@ restart_firewall (void)
 		NULL);
 /*	gchar *soundfile; */
 
-	if (geteuid ()) {
-		GtkWidget *dialog=gnome_error_dialog (_(
-			"You must be root to manipulate the firewall."));
-		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
+	if (root_alert ())
 		return;
-	}
 
 	system (commandline);
 	g_print (_("Firewall script restarted\n"));
@@ -245,12 +262,8 @@ halt_traffic (void)
 	gchar *sbin;
 	gchar *command;
 
-	if (geteuid ()) {
-		GtkWidget *dialog=gnome_error_dialog (_(
-			"You must be root to manipulate the firewall."));
-		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
+	if (root_alert ())
 		return;
-	}
 
 	if (gnome_config_get_bool ("/firestarter/Druid/locatesbins=TRUE"))
 		if (NETFILTER)
@@ -394,10 +407,13 @@ show_about (GtkWidget *widget, gpointer data)
 void
 set_panel_pic (gchar *pixmap[])
 {
-	gtk_container_remove (GTK_CONTAINER (panelvbox), panelpic);
-	panelpic = gnome_pixmap_new_from_xpm_d (pixmap);
-	gtk_box_pack_end (GTK_BOX (panelvbox), panelpic, FALSE, FALSE, 2);
-	gtk_widget_show (panelpic);
+	if( !NO_APPLET )
+	{
+		gtk_container_remove (GTK_CONTAINER (panelvbox), panelpic);
+		panelpic = gnome_pixmap_new_from_xpm_d (pixmap);
+		gtk_box_pack_end (GTK_BOX (panelvbox), panelpic, FALSE, FALSE, 2);
+		gtk_widget_show (panelpic);
+	}
 }
 
 /* [ status_docklet_build_plug ]
@@ -519,9 +535,22 @@ main (int argc, char* argv[])
 	CORBA_Environment ev;
 	CORBA_ORB orb;
 
+	/* command line stuff */
+	poptContext ctx;
+	gint opt_up, opt_down, opt_stop;
+	
+	const struct poptOption options[] =
+	{
+		{"noapplet", 'a', POPT_ARG_NONE, &NO_APPLET, 0, _("Startup without panel applet"), NULL},
+		{"start", 'u', POPT_ARG_NONE, &opt_up, 0, _("Start the firewall"), NULL},
+		{"stop", 'd', POPT_ARG_NONE, &opt_down, 0, _("Stop the firewall"), NULL},
+		{"halt", 'h', POPT_ARG_NONE, &opt_stop, 0, _("Halt all network traffic"), NULL},
+		{NULL, 0, 0, NULL, 0, NULL, NULL}
+	};
+	
 	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
 	textdomain (PACKAGE);
-
+	
 	hitlogtitles[0] = _("Port");
 	hitlogtitles[1] = _("Sent from");
 	hitlogtitles[2] = _("Service");
@@ -532,19 +561,37 @@ main (int argc, char* argv[])
 	allowsmtitles[0] = _("Open service to machine");
 	allowsatitles[0] = _("Open service to anyone");
 
+	NETFILTER = detect_netfilter ();
+
+	opt_up = opt_down = opt_stop = 0;
+	ctx = poptGetContext (NULL, argc, argv, options, 0);
+	poptGetNextOpt( ctx );
+	poptResetContext (ctx);
+
+	if (opt_up || opt_down || opt_stop)
+	{
+		CONSOLE = TRUE;
+		NO_APPLET = TRUE;
+
+		if (opt_up)
+			restart_firewall ();
+		else if (opt_down)
+			stop_firewall ();
+		else if (opt_stop)
+			halt_traffic ();
+	
+		return 0;
+	}
+
 	CORBA_exception_init (&ev);
-	orb = gnome_CORBA_init (PACKAGE, VERSION, &argc, argv, 0, &ev);
-
-
+	orb = gnome_CORBA_init_with_popt_table (PACKAGE, VERSION, &argc, argv, options, 0, &ctx, 0, &ev);
+	
 	/* Create the main window, append hostname to title */
 	if (!gethostname (hostname, 39))
 		window = gnome_app_new (PACKAGE, g_strconcat ("Firestarter ", hostname, NULL));
 	else
 		window = gnome_app_new (PACKAGE, "Firestarter");
 
-
-	NETFILTER = detect_netfilter ();
-
 	ttips = gtk_tooltips_new ();
 
 /* Set up the session managment */
@@ -555,11 +602,15 @@ main (int argc, char* argv[])
 			    GTK_SIGNAL_FUNC (session_die), NULL);
 
 /* Set up the docklet */
-	status_dock = status_docklet_new ();
-	gtk_signal_connect (GTK_OBJECT (status_dock), "build_plug",
-			    GTK_SIGNAL_FUNC (status_docklet_build_plug), NULL);
-	status_docklet_run (STATUS_DOCKLET (status_dock));
-
+	if (!NO_APPLET)
+	{
+		status_dock = status_docklet_new ();
+		gtk_signal_connect (GTK_OBJECT (status_dock), "build_plug",
+				    GTK_SIGNAL_FUNC (status_docklet_build_plug), NULL);
+		status_docklet_run (STATUS_DOCKLET (status_dock));
+	}
+	else
+		status_dock = NULL;
 /* Set up the window */
 	gtk_signal_connect (GTK_OBJECT (window), "delete_event",
 			    GTK_SIGNAL_FUNC (closewindow), NULL);
